/*
    This file is part of Escota.

    Escota is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Escota is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Escota.  If not, see <http://www.gnu.org/licenses/>.
 */


package escota.flickr;
import com.aetrion.flickr.FlickrException;
import com.aetrion.flickr.photos.PhotoList;
import com.aetrion.flickr.photos.Photo;
import com.aetrion.flickr.tags.Tag;
import escota.EscotaTaskGenerator;
import java.io.IOException;
import java.io.Serializable;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import org.xml.sax.SAXException;

/**
 *
 * @author AGuS
 */
public class Rule implements Serializable {

    /********************************************/
    /* Conditions that have to be satisfied */
    /********************************************/
    /* The photo visits number has to be greater o equal to minViews */
    private int minViews;
    /* The photo favorites number has to be greater o equal to minFabs */
    private int minFavs;
    /* The photo has to have all of the tag list */
    private List<String> tags;
    /* The photo comments number has to be greater o equal to minComments */
    private int minComments;
    /* Dates between the photo had to be taken */
    private Date dateTaken1, dateTaken2;
    /* Dates between the photo had to be posted */
    private Date datePosted1, datePosted2;
    private String name;

    private int maxViews;
    private int maxFavs;

    private String description;
    private String groupToAddName;
    private String groupToDeleteName;

    /* If all the conditons was true we are adding the photo into this group */
    private String groupIDToAdd;
    private String groupIDToDelete;

    /**
     *
     * @param photo Photo to check the rule
     * @return Returns true if photo satisfied the rule
     * @throws java.io.IOException
     * @throws org.xml.sax.SAXException
     * @throws com.aetrion.flickr.FlickrException
     */
    public boolean checkRule(Photo photo, Conexion conexion) throws IOException, SAXException, FlickrException
    {
        boolean res = (photo.getComments() >= minComments);
        res &= (photo.getViews() >= minViews) && (photo.getViews() <= maxViews);
        res &= (conexion.numberFavorites(photo) >= minFavs) && (conexion.numberFavorites(photo) <= minFavs);

        if(datePosted1 != null)
            res &= photo.getDatePosted().after(datePosted1);
        if(datePosted2 != null)
            res &= photo.getDatePosted().before(datePosted2);
        if(dateTaken1 != null)
            res &= photo.getDateTaken().after(dateTaken1);
        if(dateTaken2 != null)
            res &= photo.getDateTaken().before(dateTaken2);

        if(tags!=null)
        {
            res &= areAllThisTagsIn(tags, photo);
        }

        return res;
    }

    private boolean areAllThisTagsIn(List<String> tags, Photo photo)
    {
        Boolean res = true;
        Iterator it = tags.iterator();
        while(it.hasNext())
        {
            String t = (String) it.next();
            if(!isTagIn(t,photo))
                return false;
        }
        return res;
    }
    private boolean isTagIn(String t, Photo photo)
    {
        boolean res = false;
        Iterator it = photo.getTags().iterator();
        while(it.hasNext())
        {
            Tag photoTag = (Tag) it.next();
            if(t.equalsIgnoreCase(photoTag.getValue()))
                return true;
        }

        return res;
    }

    /**
     *
     * @param photos for each photo in photos if satfisfied the rule create a task in a queue
     * @return  Returns the queue of tasks that has been generated
     * @throws java.io.IOException
     * @throws org.xml.sax.SAXException
     * @throws com.aetrion.flickr.FlickrException
     */
    public TaskQueue generateTasks(PhotoList photos, Conexion conexion, EscotaTaskGenerator tg) throws IOException, SAXException, FlickrException
    {
        TaskQueue taskQueue = new TaskQueue(conexion.getNsid());
        Iterator it = photos.iterator();
        int i = 0;
        tg.getProgressBar().setMinimum(i);
        tg.getProgressBar().setMaximum(photos.size());



        while(it.hasNext())
        {
            tg.getProgressBar().setValue(i);
            Photo photo = (Photo) it.next();

            if(checkRule(photo, conexion))
            {
                Task task = null;
                //Crear tarea de remove
                if(this.groupIDToAdd != null)
                {
                    task = new Task(photo.getId(), photo.getTitle(), groupIDToAdd, groupToAddName, this,false);
                    taskQueue.offer(task);
                }

                //Crear tarea de add
                if(this.groupIDToDelete != null)
                {
                    task = new Task(photo.getId(), photo.getTitle(), this.groupIDToDelete, this.groupToDeleteName, this, true);
                    taskQueue.offer(task);
                }
            }
            i++;
        }

        return taskQueue;

    }

     /**
     * Create a new Rule
     */
    public Rule(String ruleName)
    {
        initConditions();
        setName(ruleName);
    }

    private void initConditions()
    {
        minViews = 0;
        maxViews = Integer.MAX_VALUE;
        maxFavs = Integer.MAX_VALUE;
        minFavs = 0;
        tags = null;
        minComments = 0;
        dateTaken1 = null;
        dateTaken2 = null;
        datePosted1 = null;
        datePosted2 = null;
        groupIDToAdd = "";
        groupIDToDelete = "";
        groupToAddName = "";
    }

    /**
     *
     * @return The photo to satisfied the rule had to be posted after this value
     */
    public Date getDatePosted1() {
        return datePosted1;
    }

    /**
     *
     * @param datePosted1 The photo to satisfied the rule had to be posted after this parameter
     */
    public void setDatePosted1(Date datePosted1) {
        this.datePosted1 = datePosted1;
    }

    /**
     *
     * @return The photo to satisfied the rule had to be posted before this value
     */
    public Date getDatePosted2() {
        return datePosted2;
    }

    /**
     *
     * @param datePosted2 : The photo to satisfied the rule had to be posted before this parameter
     */
    public void setDatePosted2(Date datePosted2) {
        this.datePosted2 = datePosted2;
    }

    /**
     *
     * @return The photo to satisfied the rule had to be taken after this value
     */
    public Date getDateTaken1() {
        return dateTaken1;
    }

    /**
     *
     * @param dateTaken1 The photo to satisfied the rule had to be taken after this parameter
     */
    public void setDateTaken1(Date dateTaken1) {
        this.dateTaken1 = dateTaken1;
    }

    /**
     *
     * @return The photo to satisfied the rule had to be taken before this value
     */
    public Date getDateTaken2() {
        return dateTaken2;
    }

    /**
     *
     * @param dateTaken2 The photo to satisfied the rule had to be taken before this parameter
     */
    public void setDateTaken2(Date dateTaken2) {
        this.dateTaken2 = dateTaken2;
    }

    /**
     *
     * @return Returns the GroupID that we are adding the picture if satisfied the rule
     */
    public String getGroupIDToAdd() {
        return groupIDToAdd;
    }

    /**
     *
     * @param groupIDToAdd the GroupID that we are adding the picture if satisfied the rule
     */
    public void setGroupToAdd(String groupID, String groupName) {
        this.groupIDToAdd = groupID;
        this.groupToAddName = groupName;
    }
    public void setGroupToDel(String groupID, String groupName) {
        this.groupIDToDelete = groupID;
        this.groupToDeleteName = groupName;
    }

    /**
     *
     * @return The photo comments number has to be greater o equal to this value
     */
    public int getMinComments() {
        return minComments;
    }

    /**
     *
     * @param minComments The photo comments number has to be greater o equal to this parameter
     */
    public void setMinComments(int minComments) {
        this.minComments = minComments;
    }

    /**
     *
     * @return The photo favorites number has to be greater o equal to this value
     */
    public int getMinFavs() {
        return minFavs;
    }

    /**
     *
     * @param minFavs The photo favorites number has to be greater o equal to this parameter
     */
    public void setMinFavs(int minFavs) {
        this.minFavs = minFavs;
    }

    /**
     *
     * @return The photo visits number has to be greater o equal to this value
     */
    public int getMinViews() {
        return minViews;
    }

    /**
     *
     * @param minViews The photo visits number has to be greater o equal to this parameter
     */
    public void setMinViews(int minViews) {
        this.minViews = minViews;
    }

    /**
     *
     * @return The photo has to have all of this tag list
     */
    public List<String> getTags() {
        return tags;
    }

    /**
     *
     * @param tags The photo has to have all of the tag list
     */
    public void setTags(List<String> tags) {
        this.tags = tags;
    }
    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }
    
    public String getGroupNameToAdd() {
        return groupToAddName;
    }
    public String getGroupNameToDel() {
        return this.groupToDeleteName;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getMaxFavs() {
        return maxFavs;
    }

    public void setMaxFavs(int maxFavs) {
        this.maxFavs = maxFavs;
    }

    public int getMaxViews() {
        return maxViews;
    }

    public void setMaxViews(int maxViews) {
        this.maxViews = maxViews;
    }


    @Override
    public String toString()
    {
        return getName();
    }
}
